import h2o_mlops_client as mlops

# Defines MLOps API URL. The address for the MLOps Gateway component.
# Usually: "https://api.mlops.my.domain"
MLOPS_API_URL = ""

# Defines IDP's token endpoint URL. For deployments using Keycloak as IDP:
# https://mlops.keycloak.domain/auth/realms/[fill-in-realm-name]/protocol/openid-connect/token
TOKEN_ENDPOINT_URL = ""

# Defines user's refresh token.
REFRESH_TOKEN = ""

# Sets client id for authentication.
CLIENT_ID = ""

# Sets client secret for authentication.
CLIENT_SECRET = ""

# Sets project id. 
PROJECT_ID = ""

# Sets registered model name. 
REGISTERED_MODEL_NAME = ""


def _get_filter_by(type, value):
    return mlops.StorageFilterRequest(
        query=mlops.StorageQuery(
            clause=[
                mlops.StorageClause(
                    property_constraint=[
                        mlops.StoragePropertyConstraint(
                            _property=mlops.StorageProperty(
                                field=type
                            ),
                            operator=mlops.StorageOperator.EQUAL_TO,
                            value=mlops.StorageValue(
                                string_value=value
                            ),
                        )
                    ]
                )
            ]
        )
    )


def _get_sort_by(field, order=mlops.ModelregistryOrder.DESCENDING):
    return mlops.StorageSortingRequest(
        _property=[
            mlops.StorageSortProperty(
                _property=mlops.StorageProperty(
                    field=field
                ),
                order=order,
            )
        ]
    )


# List all the registered models for the given project.
def list_registered_models(
    mlops_client: mlops.Client, 
    list_model_request: mlops.StorageListRegisteredModelsRequest
):
    return mlops_client.storage.registered_model.list_registered_models(list_model_request)


# Models can be filtered by parameters such as registered model name, created_time, last_modified_time, 
# updated_by, owner_id and description. This example uses the name parameter.
def get_registered_model_list_request(project_id):
    return mlops.StorageListRegisteredModelsRequest(
        project_id=project_id,
        paging=mlops.StoragePagingRequest(),
        filter=_get_filter_by("name", REGISTERED_MODEL_NAME),
        sorting=_get_sort_by("last_modified_time")
    )


def main():
    # Set up the token provider using an existing refresh token.
    mlops_token_provider = mlops.TokenProvider(
        refresh_token=REFRESH_TOKEN,
        client_id=CLIENT_ID,
        client_secret=CLIENT_SECRET,
        token_endpoint_url=TOKEN_ENDPOINT_URL,
    )

    # Set up the MLOps client.
    mlops_client = mlops.Client(
        gateway_url=MLOPS_API_URL,
        token_provider=mlops_token_provider,
    )

    list_model_response: mlops.StorageListRegisteredModelsResponse = list_registered_models(
        mlops_client,
            get_registered_model_list_request(
                project_id=PROJECT_ID
                )
            )
    print(list_model_response)


if __name__ == "__main__":
    main()
